/*
 SerialToLCD.ino
 
 This demonstrates the use of a POLLIN UC121902 TNARX LCD display with a KS0035 LCD-Driver.
 The UC121902 library should works with all LCD displays that are compatible with the 
 KS0035 driver.
 
 1 = +5 V
 2 = GND
 3 = Chip enable	Arduino Pin 7
 4 = CLK		Arduino Pin 6
 5 = Data in		Arduino Pin 5
 
 This sketch displays text sent over the serial port 
 Open the Serial-Monitor and type in what you want.
 
 created 18 Nov 2014 
 by Alexander Roehl
 
 */

// include the library
#include <UC121902.h>

void setup() {
  // initialize the serial communications
  Serial.begin(9600);
  // initialize the library with the numbers of the Arduino pins
  UC121902.begin(7,6,5);
  // clear the screen
  UC121902.clear();
}


void loop() {
  // when characters arrive over the serial port...
  if (Serial.available() > 0) {
    // clear the screen
    UC121902.clear();

    // read all the available characters and numbers
    while (Serial.available() > 0) {
      // read the incomming character or number
      int val = Serial.read();

      // the ASCII value must be changed into the correct number 
      // because the incoming number is a character
      if (val >= 48 && val <= 57) {
        // display character as a number (the ASCII-Code for 0 is 48)
        UC121902.print(val-48);
      }
      else {
        // otherwise display character to the LCD
        UC121902.print(char(val));
      }
    }
  } 
}

