/*

 0                                                                    111
 |---------------------------------------------------------------------|
 0                    52|53       55|56                 108|109       111
 |----------------------|-----------|----------------------|-----------| 
 
 BUFFER_BYTE[0] 0b00000000 digit 1
 BUFFER_BYTE[1] 0b00000000 digit 2
 BUFFER_BYTE[2] 0b00000000 digit 3
 BUFFER_BYTE[3] 0b00000000 digit 4
 BUFFER_BYTE[4] 0b00000000 digit 5
 BUFFER_BYTE[5] 0b00000000 digit 6
 BUFFER_BYTE[6] 0b00000000 symbols and function / mode bits
 
 BUFFER_BYTE[7] 0b00000000 digit 7
 BUFFER_BYTE[8] 0b00000000 digit 8
 BUFFER_BYTE[9] 0b00000000 digit 9
 BUFFER_BYTE[10] 0b00000000 digit 10
 BUFFER_BYTE[11] 0b00000000 digit 11
 BUFFER_BYTE[12] 0b00000000 digit 12
 BUFFER_BYTE[13] 0b00000000 symbols and latch bit
 
 
 created 15 Nov 2014
 by Alexander Roehl
 
 */

#include "UC121902.h"
#include "Arduino.h"
#include <inttypes.h>

LCD UC121902;


//**************************************************
// Hardware initialization
//**************************************************

void LCD::begin(int ce, int clk, int data) {
  CE = ce;
  CLK = clk;
  DATA = data;
  pinMode(ce, OUTPUT);
  pinMode(clk, OUTPUT);
  pinMode(data, OUTPUT);

  BUFFER_BYTE[6] = DP;
  BUFFER_BYTE[13] = DR;
}


//**************************************************
// LCD Communication
//**************************************************

void LCD::send() {  
  digitalWrite(CE, 1);
  // gets the buffer bytes
  for (int a=0; a<14; a++) {
	// reads the bits from right to left
    for (int b=7; b>=0; b--) {
      digitalWrite(CLK, 0);
      delayMicroseconds(TIME);
	  // sends the bits
      digitalWrite(DATA, 1 & (BUFFER_BYTE[a]>>b));
      //Serial.print(1 & (BUFFER_BYTE[a]>>b));
      delayMicroseconds(TIME);
      digitalWrite(CLK, 1); 
      delayMicroseconds(CLK_TIME);
    }
	// instruction for the 2. part (duty 1/2) 
    if (a==6) {
      digitalWrite(CE, 0);
      delayMicroseconds(1.25);
      digitalWrite(CE, 1);
      //Serial.print("\n-----");
    }
    //Serial.println();
  }
  digitalWrite(CE, 0);
}


//**************************************************
// LCD functions
//**************************************************

void LCD::clear() {
  // sets all bytes to 0
  for (int a=0;a<13;a++) {
    BUFFER_BYTE[a] = 0x00;
  } 
  // reset mode and latch
  BUFFER_BYTE[6] = DP;
  BUFFER_BYTE[13] = DR;
  cursorPos = 0;
  send();
}

void LCD::setCursor(int pos) {
  cursorPos = pos;
}

void LCD::print(int num) {
	int digit[12];
	int a;

	if (num == 0) {
		BUFFER_BYTE[CURSOR_POS[cursorPos]] = INT[0];
		cursorPos++;
	}
	else {
		// splits the number into single numbers and writes it in the digit array
		while (num > 0)
		{
			digit[a] = num%10;
			num /= 10;
			a++;
		}
		// writes the single numbers to the buffer
		while (a > 0)
		{
			a--;
			BUFFER_BYTE[CURSOR_POS[cursorPos]] = INT[digit[a]];
			cursorPos++;
		}
	}
	send();
}

void LCD::print(long num) {
  int digit[12];
  int a;

  if (num == 0) {
    BUFFER_BYTE[CURSOR_POS[cursorPos]] = INT[0]; 
    cursorPos++;
  }
  else {
	// splits the number into single numbers and writes it in the digit array
    while (num > 0)
    {
      digit[a] = num%10;
      num /= 10;
      a++;
    }
	// writes the single numbers to the buffer
    while (a > 0)
    {
      a--;
      BUFFER_BYTE[CURSOR_POS[cursorPos]] = INT[digit[a]]; 
      cursorPos++;
    }  
  }
  send();
}

void LCD::print(char chr) {
  for (int a=0; a<47; a++) {
    if (chr == CHAR[a][0])
      BUFFER_BYTE[CURSOR_POS[cursorPos]] = CHAR[a][1]; 
  }
  cursorPos++;
  send();
}

void LCD::print(char* chr) {
  // splits the string into single chars
  while(*chr)
  {
    print(*chr++);
  }
}


void LCD::sign(int val, int onOff) {
  if (val < 4) {
    if (onOff)
      BUFFER_BYTE[6] |= SYMBOL[val];
    else
      BUFFER_BYTE[6] ^= SYMBOL[val];
  } 
  else {
    if (onOff)
      BUFFER_BYTE[13] |= SYMBOL[val];
    else
      BUFFER_BYTE[13] ^= SYMBOL[val];
  }
  send();
}


void LCD::help() {
  Serial.println("Command\t\t\t\tDescription\n\n");
  Serial.println("void help()");
  Serial.println("void begin(ce, clk, data)");
  Serial.println("void clear()");
  Serial.println("void setCursor(int)");
  Serial.println("void print(int)");
  Serial.println("void print(long)");
  Serial.println("void print(char)");
  Serial.println("void print(char*)");
  Serial.println("void sign(int, int [ON|OFF])");
  Serial.println("    BELL");
  Serial.println("    SLASH ");
  Serial.println("    MEM ");
  Serial.println("    CHAN");
  Serial.println("    SEC");
  Serial.println("    PROG");
  Serial.println("    BAT_LOW");
  Serial.println("    BAT_FULL");

}

/*
void LCD::con(byte val) {
	BUFFER_BYTE[6] = val | DQ;
	digitalWrite(CE, 1);
	for (int a=0; a<7; a++) {
		for (int b=7; b>=0; b--) {
			digitalWrite(CLK, 0);
			delayMicroseconds(TIME);
			digitalWrite(DATA, 1 & (BUFFER_BYTE[a]>>b));
			//Serial.print(1 & (BUFFER_BYTE[a]>>b));
			delayMicroseconds(TIME);
			digitalWrite(CLK, 1);
			delayMicroseconds(CLK_TIME);
		}
	}
	digitalWrite(CE, 0);
	BUFFER_BYTE[6] = DP;
}
*/