/*

 This is a library for the POLLIN UC121902 TNARX LCD display with the KS0035 driver
 Designed specifically to work with the Arduino
 
 1 = +5 V
 2 = GND
 3 = Chip enable
 4 = CLK
 5 = Data in
 
      a
   f     b
      g 
   e     c
      d
      
  c   d b   g a e f
  1 0 1 1   1 1 1 1


 created 15 Nov 2014
 by Alexander Roehl
 
*/


#ifndef _UC121902_H_INCLUDED
#define _UC121902_H_INCLUDED

#include <stdio.h>
#include <Arduino.h>
#include <inttypes.h>

#define TIME 1
#define CLK_TIME 1

// function / mode bits
#define DP  0x04	// Drive mode
#define DQ  0x02	// AD / DSP function
#define DR  0x01	// Latch clock


// cursor position includes the array index of the BUFFER_BYTE
const int CURSOR_POS[12] = { 12, 11, 10, 9, 8, 7, 5, 4, 3, 2, 1, 0 };
	
/*******************************************************
 Address definition of the chars
 Not all letters are available. (M, V, W, X)
 Available characters:
 ABCDEFGHIJKLNOPQRSTUYZ=-
********************************************************/
const byte CHAR[47][2] = { 
  {'A',0x9f},   
  {'a',0x9f}, 
  {'B',0xbf}, 
  {'b',0xab}, 
  {'C',0x27}, 
  {'c',0x2a}, 
  {'D',0xba}, 
  {'d',0xba}, 
  {'E',0x2f}, 
  {'e',0x2f}, 
  {'F',0x0f}, 
  {'f',0x0f}, 
  {'G',0xaf}, 
  {'g',0xbd}, 
  {'H',0x9b}, 
  {'h',0x8b}, 
  {'I',0x90}, 
  {'i',0x90}, 
  {'J',0xb0}, 
  {'j',0xb0}, 
  {'K',0x9b}, 
  {'k',0x9b}, 
  {'L',0x23}, 
  {'l',0x23}, 
  {'N',0x97}, 
  {'n',0x8a}, 
  {'O',0xb7}, 
  {'o',0xaa}, 
  {'P',0x1f}, 
  {'p',0x1f}, 
  {'Q',0x9d}, 
  {'q',0x9d}, 
  {'R',0x0a}, 
  {'r',0x0a}, 
  {'S',0xad}, 
  {'s',0xad}, 
  {'T',0x2b}, 
  {'t',0x2b}, 
  {'U',0xb3}, 
  {'u',0xa2}, 
  {'Y',0x99}, 
  {'y',0x99}, 
  {'Z',0x36}, 
  {'z',0x36}, 
  {' ',0x00}, 
  {'=',0x28}, 
  {'-',0x08} 
};

// Address definition of the numbers 
const byte INT[10] = { 0xb7, 0x90,  0x3e, 0xbc, 0x99, 0xad, 0xaf, 0x94, 0xbf, 0xbd };
// Address definition of the symbols
const byte SYMBOL[8] = { 0x80, 0x40, 0x20, 0x10, 0x80, 0x40, 0x20, 0x10 };
#define BELL        0  // 0x80
#define SLASH       1  // 0x40
#define MEM         2  // 0x20
#define CHAN        3  // 0x10
#define SEC         4  // 0x80
#define PROG        5  // 0x40
#define BAT_LOW     6  // 0x20
#define BAT_FULL    7  // 0x10
#define ON 1
#define OFF 0

class LCD {
  private:
    byte BUFFER_BYTE[14];
    void send();
    int cursorPos;
  public:
    int CE;		// Chip enable pin
	int CLK;	// Clock pin
	int DATA;	// Data pin
	//void con(byte);
    void help(void);
    void begin(int,int,int);
    void clear(void);
    void setCursor(int);
    void print(int);
	void print(long);
    void print(char);
    void print(char*);
    void sign(int, int);
};

extern LCD UC121902;

#endif
